<?php

namespace Drupal\semantic_search\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;

/**
 * Semantic Search configuration form.
 */
class SemanticSearchIndexingForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'semantic_search.indexing',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'semantic_search_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $embedding_service = \Drupal::service('semantic_search.openai_embedding');
    $saved_embeddings = $embedding_service->getSavedEmbeddings();

    $form['help'] = [
      '#type' => 'item',
      '#title' => $this->t('Indexed status'),
      '#markup' => $this->t('@indexed nodes', ['@indexed' => count($saved_embeddings)]),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Index site'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $query = \Drupal::entityQuery('node');
    // @todo multilingue.
    $query->condition('status', NodeInterface::PUBLISHED);
    $nodeIds = $query->execute();
    $nodes = Node::loadMultiple($nodeIds);

    $counter = 0;
    $batch = [
      'operations' => [],
      'title' => $this->t('Indexing Nodes'),
      'finished' => 'SemanticSearchIndexingForm::indexingFinished',
    ];

    foreach ($nodes as $node) {
      $batch['operations'][] = [
        [$this, 'bulkIndexing'],
        [$node->id()],
      ];
      $counter++;
    }

    if ($counter > 0) {
      batch_set($batch);
      $this->messenger()->addStatus($this->t('Indexing launched for @counter Nodes.', ['@counter' => $counter]));
    }
    else {
      $this->messenger()->addWarning($this->t('Nothing to index.'));
    }

  }

  /**
   * Export the quotation.
   */
  public function bulkIndexing($nid, &$context) {

    $node = \Drupal::entityTypeManager()->getStorage('node')->load($nid);
    $embedding_service = \Drupal::service('semantic_search.openai_embedding');
    $saved_embeddings = $embedding_service->getSavedEmbeddings();

    $builder = \Drupal::entityTypeManager()->getViewBuilder('node');
    $build = $builder->view($node);
    $node_content = \Drupal::service('renderer')->render($build);
    $embedding = $embedding_service->getEmbedding($node_content);

    if ($embedding) {
      $saved_embeddings[$node->id()] = $embedding;
      $embedding_service->saveEmbeddings($saved_embeddings);
    }

    $context['results']['count']++;
  }

  /**
   * Batch Finished callback.
   *
   * @param bool $success
   *   Success of the operation.
   * @param array $results
   *   Array of results for post processing.
   * @param array $operations
   *   Array of operations.
   */
  public static function indexingFinished($success, array $results, array $operations) {
    if ($success) {

      $message = t('@exported_text', [
        '@exported_text' => t('@count Nodes Indexed', ['@count' => $results['count']]),
      ]);
      \Drupal::messenger()->addStatus($message);
    }
    else {
      $message = t('An error has occurred. Please contact an administrator.');
      \Drupal::messenger()->addError($message);
    }

  }

  /**
   * {@inheritdoc}
   */
  protected function cleanRenderedField($content) {
    $content = strip_tags($content);
    $content = preg_replace('~[\r\n]+~', '', $content);
    return $content;
  }

}
